@@ -44,6 +44,7 @@
 #include "plugins.h"
 #include "../common/xchat.h"
 #include "../common/xchatc.h"
+#include "../common/dcc.h"
 #include "../common/servlist.h"
 #include "../common/fe.h"
 #include "../common/util.h"
@@ -87,7 +88,7 @@ fe_args (int argc, char *argv[])
 
 #ifdef HAVE_LIBGNOME_214
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
-	gui.program = gnome_program_init (PACKAGE, VERSION,
+	gui.program = gnome_program_init (PACKAGE_NAME, PACKAGE_VERSION,
 	                                  LIBGNOMEUI_MODULE, argc, argv,
 	                                  GNOME_PARAM_GOPTION_CONTEXT, context,
 	                                  GNOME_PARAM_HUMAN_READABLE_NAME, _("IRC Chat"),
@@ -99,7 +100,7 @@ fe_args (int argc, char *argv[])
 	g_option_context_parse (context, &argc, &argv, &error);
 	g_option_context_free (context);
 
-	gui.program = gnome_program_init (PACKAGE, VERSION,
+	gui.program = gnome_program_init (PACKAGE_NAME, PACKAGE_VERSION,
 	                                  LIBGNOMEUI_MODULE, argc, argv,
 	                                  GNOME_PARAM_HUMAN_READABLE_NAME, _("IRC Chat"),
 	                                  GNOME_PROGRAM_STANDARD_PROPERTIES,
@@ -113,7 +114,7 @@ fe_args (int argc, char *argv[])
 	}
 
 	if (opt_version) {
-		g_print ("xchat-gnome %s\n", VERSION);
+		g_print ("xchat-gnome %s\n", PACKAGE_VERSION);
 		return 0;
 	}
 
@@ -232,10 +233,19 @@ fe_new_window (struct session *sess, int focus)
 	}
 
 	conversation_panel_add_session (CONVERSATION_PANEL (gui.conversation_panel), sess, (gboolean) focus);
-	if (sess->type == SESS_SERVER)
+	if (sess->type == SESS_SERVER) {
 		navigation_tree_create_new_network_entry (gui.server_tree, sess);
-	else if (sess->type == SESS_CHANNEL || sess->type == SESS_DIALOG)
+	} else if (sess->type == SESS_CHANNEL) {
 		navigation_tree_create_new_channel_entry (gui.server_tree, sess, (gboolean) focus);
+	} else if (sess->type == SESS_DIALOG) {
+		struct User *user = NULL;
+
+		user = userlist_find_global (sess->server, sess->channel);
+
+		navigation_tree_create_new_channel_entry (gui.server_tree, sess, (gboolean) focus);
+		if (user)
+			topic_label_set_topic (TOPIC_LABEL (gui.topic_label), sess, user->hostname);
+	}
 #ifdef USE_PLUGIN
 	if (!(opt_noplugins || loaded)) {
 		loaded = TRUE;
@@ -457,22 +467,58 @@ fe_userlist_clear (struct session *sess)
 void
 fe_dcc_add (struct DCC *dcc)
 {
-	/* FIXME: chats? */
-	dcc_window_add (gui.dcc, dcc);
+	if (dcc->type == TYPE_CHATRECV || dcc->type == TYPE_CHATSEND) {
+		/* chats */
+		if (prefs.autodccchat == FALSE) {
+			GtkWidget *dialog;
+			gint response;
+
+			dialog = gtk_message_dialog_new (GTK_WINDOW (gui.main_window),
+			                                 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+			                                 GTK_MESSAGE_QUESTION,
+			                                 GTK_BUTTONS_CANCEL,
+			                                 _("Incoming DCC Chat"));
+			gtk_dialog_add_button (GTK_DIALOG (dialog),
+			                       _("_Accept"),
+			                       GTK_RESPONSE_ACCEPT);
+			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+				_("%s is attempting to create a direct chat. Do you wish to accept the connection?"),
+				dcc->nick);
+
+			response = gtk_dialog_run (GTK_DIALOG (dialog));
+			if (response == GTK_RESPONSE_ACCEPT) {
+				dcc_get (dcc);
+			} else {
+				dcc_abort (dcc->serv->server_session, dcc);
+			}
+			gtk_widget_destroy (dialog);
+		}
+	} else {
+		/* file transfers */
+		dcc_window_add (gui.dcc, dcc);
+	}
 }
 
 void
 fe_dcc_update (struct DCC *dcc)
 {
-	/* FIXME: chats? */
-	dcc_window_update (gui.dcc, dcc);
+	if (dcc->type == TYPE_CHATRECV || dcc->type == TYPE_CHATSEND) {
+		/* chats */
+	} else {
+		/* file transfers */
+		dcc_window_update (gui.dcc, dcc);
+	}
 }
 
 void
 fe_dcc_remove (struct DCC *dcc)
 {
-	/* FIXME: chats? */
-	dcc_window_remove (gui.dcc, dcc);
+	if (dcc->type == TYPE_CHATRECV || dcc->type == TYPE_CHATSEND) {
+		/* chats */
+	} else {
+		/* file transfers */
+		dcc_window_remove (gui.dcc, dcc);
+	}
 }
 
 int
@@ -917,10 +963,9 @@ save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style,
 	gint argc;
 	const gchar *prefix;
 
-	argv = g_new0 (gchar *, 3);
+	argv = g_new0 (gchar *, 2);
 	argv[0] = client_data;
-	argv[1] = "--no-auto";
-	argc = 2;
+	argc = 1;
 
 	gnome_client_set_restart_style (client, GNOME_RESTART_IF_RUNNING);
 	gnome_client_set_restart_command (client, argc, argv);
