@@ -90,6 +90,7 @@ static gboolean on_resize (GtkWidget *widget, GdkEventConfigure *event, gpointer
 static gboolean on_hpane_move (GtkPaned *widget, GParamSpec *param_spec, gpointer data);
 
 static gboolean on_main_window_focus_in (GtkWidget *widget, GdkEventFocus *event, gpointer data);
+static gboolean on_main_window_configure (GtkWidget *widget, GdkEventConfigure *event, gpointer data);
 
 static GtkActionEntry action_entries [] = {
 
@@ -152,6 +153,8 @@ initialize_main_window (void)
 	                  G_CALLBACK (on_main_window_close), NULL);
 	g_signal_connect (G_OBJECT (gui.main_window), "focus-in-event",
 	                  G_CALLBACK (on_main_window_focus_in), NULL);
+	g_signal_connect (G_OBJECT (gui.main_window), "configure-event",
+	                  G_CALLBACK (on_main_window_configure), NULL);
 
 	/* hook up the menus */
 	gui.action_group = gtk_action_group_new ("MenuAction");
@@ -180,6 +183,7 @@ initialize_main_window (void)
 	gui.find_bar           = glade_xml_get_widget (gui.xml, "find_bar");
 	gui.status_bar         = glade_xml_get_widget (gui.xml, "status_bar");
 	gui.text_entry         = glade_xml_get_widget (gui.xml, "text_entry");
+	gui.topic_hbox         = glade_xml_get_widget (gui.xml, "topic_hbox");
 	gui.topic_label        = glade_xml_get_widget (gui.xml, "topic_label");
 
 	/* Hook up accelerators for pgup/pgdn */
@@ -403,7 +407,12 @@ on_edit_cut_activate (GtkAction *action, gpointer data)
 static void
 on_edit_copy_activate (GtkAction *action, gpointer data)
 {
-	gtk_editable_copy_clipboard (GTK_EDITABLE (gui.text_entry));
+	if (gtk_editable_get_selection_bounds (GTK_EDITABLE (gui.text_entry), NULL, NULL))
+		/* There is something selected in the text_entry */
+		gtk_editable_copy_clipboard (GTK_EDITABLE (gui.text_entry));
+	else
+		/* Nothing selected, we copy from the conversation panel */
+		conversation_panel_copy_selection (CONVERSATION_PANEL (gui.conversation_panel));
 }
 
 static void
@@ -592,6 +601,12 @@ on_help_about_activate (GtkAction *action, gpointer data)
 	show_about_dialog ();
 }
 
+static void
+nickname_dialog_entry_activated (GtkEntry *entry, GtkDialog *dialog)
+{
+	gtk_dialog_response (dialog, GTK_RESPONSE_OK);
+}
+
 static void
 on_nickname_clicked (GtkButton *widget, gpointer user_data)
 {
@@ -609,6 +624,7 @@ on_nickname_clicked (GtkButton *widget, gpointer user_data)
 
 	gtk_entry_set_text (GTK_ENTRY (entry), current_sess->server->nick);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (away), current_sess->server->is_away);
+	g_signal_connect (G_OBJECT (entry), "activate", G_CALLBACK (nickname_dialog_entry_activated), dialog);
 
 	result = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (result == GTK_RESPONSE_OK) {
@@ -738,3 +754,10 @@ on_main_window_focus_in (GtkWidget * widget, GdkEventFocus * event, gpointer dat
 	gtk_window_set_urgency_hint (GTK_WINDOW (widget), FALSE);
 	return FALSE;
 }
+
+static gboolean
+on_main_window_configure (GtkWidget *widget, GdkEventConfigure *event, gpointer data)
+{
+	conversation_panel_queue_tdraw (CONVERSATION_PANEL (gui.conversation_panel));
+	return FALSE;
+}
