@@ -49,17 +49,18 @@ static void      navigation_tree_init (NavTree *navtree);
 static void      navigation_tree_class_init (NavTreeClass *klass);
 static void      navigation_tree_dispose (GObject *object);
 static void      navigation_tree_finalize (GObject *object);
-static session  *navigation_tree_get_selected_session (void);
 static void      navigation_tree_update_refs (NavTree *navtree);
 /* Context menus. */
-static void      navigation_context (GtkWidget *treeview, session *selected);
-static void      server_context (GtkWidget *treeview, session *selected);
-static void      channel_context (GtkWidget *treeview, session *selected);
-static void      dialog_context (GtkWidget *treeview, session *selected);
+static GtkWidget *navigation_context (GtkWidget *treeview, session *selected);
+static GtkWidget *server_context (GtkWidget *treeview, session *selected);
+static GtkWidget *channel_context (GtkWidget *treeview, session *selected);
+static GtkWidget *dialog_context (GtkWidget *treeview, session *selected);
 static gboolean  channel_is_autojoin (ircnet *network, gchar *chan);
+static void      show_context_menu (GtkWidget *treeview, GdkEventButton *event);
 /* Callbacks. */
 static gboolean  click (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
 static gboolean  declick (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
+static gboolean  popup_menu (GtkWidget *treeview, gpointer user_data);
 static void      navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_data);
 static void      row_expanded (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data);
 static void      row_collapsed (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data);
@@ -70,7 +71,6 @@ static void      on_server_channel_list (GtkAction *action, gpointer data);
 static void      on_save (GtkAction *action, gpointer data);
 static void      on_close (GtkAction *action, gpointer data);
 static void      on_find (GtkAction *action, gpointer data);
-static void      on_clear (GtkAction *action, gpointer data);
 static void      on_channel_leave (GtkAction *action, gpointer data);
 static void      on_channel_bans (GtkAction *action, gpointer data);
 static void      on_server_autoconnect (GtkAction *action, gpointer data);
@@ -89,22 +89,20 @@ static GtkActionEntry action_entries[] = {
 	{"ChannelLeave",      GTK_STOCK_QUIT,           N_("_Leave"),           "", NULL, G_CALLBACK (on_channel_leave)},
 	{"ChannelClose",      GTK_STOCK_CLOSE,          N_("_Close"),           "", NULL, G_CALLBACK (on_close)},
 	{"ChannelFind",       GTK_STOCK_FIND,           N_("_Find..."),         "", NULL, G_CALLBACK (on_find)},
-	{"ChannelClear",      GTK_STOCK_CLEAR,          N_("C_lear"),           "", NULL, G_CALLBACK (on_clear)},
 	{"ChannelBans",       GTK_STOCK_DIALOG_WARNING, N_("_Bans..."),         "", NULL, G_CALLBACK (on_channel_bans)},
 
 	/* Dialog context menu */
 	{"DialogSave",        GTK_STOCK_SAVE,           N_("_Save Transcript"), "", NULL, G_CALLBACK (on_save)},
 	{"DialogClose",       GTK_STOCK_CLOSE,          N_("_Close"),           "", NULL, G_CALLBACK (on_close)},
 	{"DialogFind",        GTK_STOCK_FIND,           N_("_Find..."),         "", NULL, G_CALLBACK (on_find)},
-	{"DialogClear",       GTK_STOCK_CLEAR,          N_("C_lear"),           "", NULL, G_CALLBACK (on_clear)},
 };
 
 static GtkToggleActionEntry toggle_action_entries[] = {
 	/* Server context menu */
-	{"ServerAutoConnect", NULL, N_("_Auto-connect"), "", NULL, G_CALLBACK (on_server_autoconnect), FALSE},
+	{"ServerAutoConnect", NULL, N_("_Auto-connect on startup"), "", NULL, G_CALLBACK (on_server_autoconnect), FALSE},
 
 	/* Channel context menu */
-	{"ChannelAutoJoin",   NULL, N_("_Auto-join"),    "", NULL, G_CALLBACK (on_channel_autojoin), FALSE},
+	{"ChannelAutoJoin",   NULL, N_("_Auto-join on connect"),    "", NULL, G_CALLBACK (on_channel_autojoin), FALSE},
 };
 
 static GtkActionGroup *action_group;
@@ -143,7 +141,7 @@ navigation_tree_init (NavTree *navtree)
 	GtkTreeSelection *select;
 
 	g_object_set ((gpointer) navtree, "headers-visible", FALSE, NULL);
-	navtree->current_path = NULL;
+	navtree->current_rowref = NULL;
 	navtree->model = NULL;
 	navtree->selection_changed_id = 0;
 
@@ -177,6 +175,7 @@ navigation_tree_init (NavTree *navtree)
 	g_signal_connect (G_OBJECT (navtree), "row-collapsed", G_CALLBACK (row_collapsed), NULL);
 	g_signal_connect (G_OBJECT (navtree), "button_press_event", G_CALLBACK (click), NULL);
 	g_signal_connect (G_OBJECT (navtree), "button_release_event", G_CALLBACK (declick), NULL);
+	g_signal_connect (G_OBJECT (navtree), "popup_menu", G_CALLBACK (popup_menu), NULL);
 
 	/* Initialize our row references. */
 	navtree->last_server = NULL;
@@ -206,8 +205,8 @@ static void
 navigation_tree_finalize (GObject *object)
 {
 	NavTree *navtree = (NavTree *) object;
-	gtk_tree_path_free (navtree->current_path);
-	navtree->current_path = NULL;
+	gtk_tree_row_reference_free (navtree->current_rowref);
+	navtree->current_rowref = NULL;
 }
 
 /* New NavTree. */
@@ -230,7 +229,6 @@ navigation_tree_new (NavModel *model)
 void
 navigation_tree_create_new_network_entry (NavTree *navtree, struct session *sess)
 {
-	GtkTreeIter *iter;
 	GtkWidget *menuitem, *button;
 
 	navigation_model_add_new_network (navtree->model, sess);
@@ -238,17 +236,7 @@ navigation_tree_create_new_network_entry (NavTree *navtree, struct session *sess
 	/* Select the new network. */
 	navigation_tree_select_session (navtree, sess);
 
-	/* Because we added a network it is likely that the path to the current session has changed
-	 * so we update it.
-	 */
-	iter = navigation_model_get_sorted_iter (navtree->model, gui.current_session);
-	if (iter) {
-		if (navtree->current_path)
-			gtk_tree_path_free (navtree->current_path);
-
-		navtree->current_path = gtk_tree_model_get_path (GTK_TREE_MODEL (navtree->model->sorted), iter);
-		gtk_tree_iter_free (iter);
-	}
+	/* We rely on the above select_session call triggering the selection 'changed' event to do book-keeping */
 
 	/* Our row references to the last channel and server should also be
 	 * updated.
@@ -267,60 +255,50 @@ navigation_tree_create_new_network_entry (NavTree *navtree, struct session *sess
 }
 
 void
-navigation_tree_create_new_channel_entry (NavTree *navtree, struct session *sess)
+navigation_tree_create_new_channel_entry (NavTree *navtree, struct session *sess, gboolean focus)
 {
-	GtkTreeIter *iter;
 	GtkWidget *menuitem, *button;
-	GtkTreeView *treeview;
 	ircnet *net;
 
 	navigation_model_add_new_channel (navtree->model, sess);
 
-	navigation_tree_select_session (navtree, sess);
+	if (focus) {
+		navigation_tree_select_session (navtree, sess);
 
-	/* Because we're adding a new channel it's possible that the path to the current
-	 * session has changed, so we'll update it.
-	 * XXX: We could probably add some kind of test to only update current_path
-	 *      if it's truly necessary (e.g. the new channel is on the same network as
-	 *      the old one), but do we need to bother?
-	 */
-	iter = navigation_model_get_sorted_iter (navtree->model, gui.current_session);
-	if (iter) {
-		if (navtree->current_path)
-			gtk_tree_path_free (navtree->current_path);
+		/* We rely on the above select_session call triggering the selection 'changed' event to do book-keeping */
 
-		navtree->current_path = gtk_tree_model_get_path (GTK_TREE_MODEL (navtree->model->sorted), iter);
-		gtk_tree_iter_free (iter);
+		/* Update the row refs here. */
+		navigation_tree_update_refs (navtree);
+
+		/* Set the userlist model */
+		gtk_tree_view_set_model (GTK_TREE_VIEW (gui.userlist),
+		                         GTK_TREE_MODEL (userlist_get_store (u, sess)));
+
+		topic_label_set_current (TOPIC_LABEL (gui.topic_label), sess);
+		net = sess->server->network;
+		if (net == NULL)
+			rename_main_window (NULL, sess->channel);
+		else
+			rename_main_window (net->name, sess->channel);
+
+		button = glade_xml_get_widget (gui.xml, "close discussion");
+		if (sess->type == SESS_CHANNEL)
+			gtk_widget_set_sensitive (button, TRUE);
+		menuitem = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionChangeTopicItem");
+		if (menuitem == NULL)
+			g_warning ("can't access topic change menu item");
+		else
+			gtk_widget_set_sensitive (menuitem, sess->type == SESS_CHANNEL);
+
+		/* Set our nick. */
+		set_nickname (sess->server, NULL);
 	}
-
-	/* Update the row refs here. */
-	navigation_tree_update_refs (navtree);
-
-	/* Set the userlist model */
-	treeview = GTK_TREE_VIEW (glade_xml_get_widget (gui.xml, "userlist"));
-	gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (userlist_get_store (u, sess)));
-
-	topic_label_set_current (TOPIC_LABEL (gui.topic_label), sess);
-	net = sess->server->network;
-	if (net == NULL)
-		rename_main_window (NULL, sess->channel);
-	else
-		rename_main_window (net->name, sess->channel);
-
-	button = glade_xml_get_widget (gui.xml, "close discussion");
-	if (sess->type == SESS_CHANNEL)
-		gtk_widget_set_sensitive (button, TRUE);
-	menuitem = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionChangeTopicItem");
-	if (menuitem == NULL)
-		g_warning ("can't access topic change menu item");
-	else
-		gtk_widget_set_sensitive (menuitem, sess->type == SESS_CHANNEL);
 }
 
 void
 navigation_tree_remove_channel (NavTree *navtree, struct session *sess)
 {
-	GtkTreePath *path = gtk_tree_path_copy (navtree->current_path);
+	GtkTreePath *path = gtk_tree_row_reference_get_path (navtree->current_rowref);
 	GtkTreeSelection *select = gtk_tree_view_get_selection (GTK_TREE_VIEW (navtree));
 	GtkTreeIter iter;
 	GtkTreeModel *sorted;
@@ -343,13 +321,7 @@ navigation_tree_remove_channel (NavTree *navtree, struct session *sess)
 	gtk_tree_selection_select_path (select, path);
 	navigation_model_remove (navtree->model, sess);
 
-	/* Make sure that the navtree's current_path is valid. If it selected the next channel
- 	 * in the list because there was no previous channel, current_path will be one item
- 	 * too high. In all other cases this is extraneous.
- 	 */
-	gtk_tree_path_free (navtree->current_path);
-	gtk_tree_selection_get_selected (select, &sorted, &iter);
-	navtree->current_path = gtk_tree_model_get_path (sorted, &iter);
+	/* We rely on the above select_path call triggering the selection 'changed' event to do book-keeping */
 
 	navigation_tree_update_refs (navtree);
 
@@ -359,7 +331,7 @@ navigation_tree_remove_channel (NavTree *navtree, struct session *sess)
 void
 navigation_tree_remove_server (NavTree *navtree, struct session *sess)
 {
-	GtkTreePath *path = gtk_tree_path_copy (navtree->current_path);
+	GtkTreePath *path = gtk_tree_row_reference_get_path (navtree->current_rowref);
 	GtkTreeSelection *select = gtk_tree_view_get_selection (GTK_TREE_VIEW (navtree));
 
 	if (!gtk_tree_path_prev (path)) {
@@ -374,11 +346,7 @@ navigation_tree_remove_server (NavTree *navtree, struct session *sess)
 			gtk_tree_selection_select_iter (select, &iter);
 			navigation_model_remove (navtree->model, sess);
 
-			/* After removing the first server the navtree's current_path
-			 * is invalid so we set it to the root of the tree.
-			 */
-			gtk_tree_path_free (navtree->current_path);
-			navtree->current_path = gtk_tree_path_new_first ();
+			/* We rely on the above select_iter call triggering the selection 'changed' event to do book-keeping */
 		} else {
 			/* The first server is the only server. */
 			GtkTreeModel *store = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (sorted));
@@ -709,12 +677,12 @@ navigation_tree_select_next_network (NavTree *navtree)
 	/* Try to get the current selection. */
 	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
 		model = gtk_tree_view_get_model (GTK_TREE_VIEW (navtree));
-		if (navtree->current_path) {
-			/* If nothing is selected and we have a current_path set the iter to that path. */
-			path = gtk_tree_path_copy (navtree->current_path);
+		if (navtree->current_rowref) {
+			/* If nothing is selected and we have a current_rowref set the iter to that path. */
+			path = gtk_tree_row_reference_get_path (navtree->current_rowref);
 			gtk_tree_model_get_iter (model, &iter, path);
 		} else {
-			/* If we have no current_path and nothing selected select the first server and return. */
+			/* If we have no current_rowref and nothing selected select the first server and return. */
 			gtk_tree_model_get_iter_first (model, &iter);
 			gtk_tree_selection_select_iter (selection, &iter);
 			return;
@@ -730,6 +698,9 @@ navigation_tree_select_next_network (NavTree *navtree)
 		gtk_tree_model_iter_parent (model, &parent, &iter);
 		iter = parent;
 	}
+
+	gtk_tree_path_free(path);
+
 	if (!gtk_tree_model_iter_next (model, &iter)) {
 		/* If we can't move to the next we need to move back to the root. Move iter to the root. */
 		GtkTreeIter root;
@@ -756,9 +727,9 @@ navigation_tree_select_prev_network (NavTree *navtree)
 		/* If there is nothing selected in the GtkTreeSelection... */
 		/* Get the model. */
 		model = gtk_tree_view_get_model (GTK_TREE_VIEW (navtree));
-		if (navtree->current_path) {
-			/* If there's a current_path set path to that. */
-			path = gtk_tree_path_copy (navtree->current_path);
+		if (navtree->current_rowref) {
+			/* If there's a current_rowref set path to that. */
+			path = gtk_tree_row_reference_get_path (navtree->current_rowref);
 		} else {
 			/* Otherwise set path to the root. */
 			path = gtk_tree_path_new_from_string ("0");
@@ -787,21 +758,25 @@ navigation_tree_select_prev_network (NavTree *navtree)
 
 	/* Select the path in our GtkTreeSelection. */
 	gtk_tree_selection_select_path (selection, path);
+	gtk_tree_path_free(path);
 }
 
-static session*
-navigation_tree_get_selected_session (void)
+session*
+navigation_tree_get_selected_session (gboolean *connected)
 {
 	GtkTreeView *treeview;
 	GtkTreeSelection *select;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	session *s;
+	gboolean temp;
 
 	treeview = GTK_TREE_VIEW (gui.server_tree);
 	select = gtk_tree_view_get_selection (treeview);
 	if (gtk_tree_selection_get_selected (select, &model, &iter)) {
-		gtk_tree_model_get (model, &iter, 2, &s, -1);
+		gtk_tree_model_get (model, &iter, 2, &s, 6, &temp, -1);
+		if (connected)
+			*connected = temp;
 		return s;
 	}
 	return NULL;
@@ -924,30 +899,29 @@ navigation_tree_server_is_connected (NavTree *navtree, gchar *name)
 
 
 /***** Context Menus *****/
-static void
+static GtkWidget *
 navigation_context (GtkWidget *treeview, session *selected)
 {
 	switch (selected->type) {
 	case SESS_SERVER:
-		server_context (treeview, selected);
-		return;
+		return server_context (treeview, selected);
 	case SESS_CHANNEL:
-		channel_context (treeview, selected);
-		return;
+		return channel_context (treeview, selected);
 	case SESS_DIALOG:
-		dialog_context (treeview, selected);
-		return;
+		return dialog_context (treeview, selected);
+	default:
+		g_return_val_if_reached (NULL);
 	}
 }
 
-static void
+static GtkWidget *
 server_context (GtkWidget *treeview, session *selected)
 {
 	GtkWidget *menu;
 	GtkAction *action;
 
 	menu = gtk_ui_manager_get_widget (gui.manager, "/ServerPopup");
-	g_return_if_fail (menu != NULL);
+	g_return_val_if_fail (menu != NULL, NULL);
 
 	if (((ircnet *)selected->server->network) == NULL) {
 		/* disable the auto-connect action.
@@ -965,11 +939,10 @@ server_context (GtkWidget *treeview, session *selected)
 		else
 			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
 	}
-
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
+	return menu;
 }
 
-static void
+static GtkWidget *
 channel_context (GtkWidget *treeview, session *selected)
 {
 	GtkWidget *menu;
@@ -977,7 +950,7 @@ channel_context (GtkWidget *treeview, session *selected)
 	ircnet *network;
 
 	menu = gtk_ui_manager_get_widget (gui.manager, "/ChannelPopup");
-	g_return_if_fail (menu != NULL);
+	g_return_val_if_fail (menu != NULL, NULL);
 
 	/* check if the channel is in the auto-join list */
 	action = gtk_action_group_get_action (action_group, "ChannelAutoJoin");
@@ -993,18 +966,54 @@ channel_context (GtkWidget *treeview, session *selected)
 			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
 	}
 
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
+	return menu;
 }
 
-static void
+static GtkWidget *
 dialog_context (GtkWidget *treeview, session *selected)
 {
 	GtkWidget *menu;
 
 	menu = gtk_ui_manager_get_widget (gui.manager, "/DialogPopup");
+	g_return_val_if_fail (menu != NULL, NULL);
+
+	return menu;
+}
+
+static void
+show_context_menu (GtkWidget *treeview, GdkEventButton *event)
+{
+	GtkTreePath *path;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GtkWidget *menu;
+	session *s;
+
+	if (NAVTREE (treeview)->current_rowref == NULL)
+		return;
+
+	model = gtk_tree_row_reference_get_model (NAVTREE (treeview)->current_rowref);
+	path = gtk_tree_row_reference_get_path (NAVTREE (treeview)->current_rowref);
+	gtk_tree_model_get_iter (model, &iter, path);
+	gtk_tree_path_free(path);
+	gtk_tree_model_get (model, &iter, 2, &s, -1);
+
+	if (s == NULL)
+		return;
+
+	menu = navigation_context (treeview, s); /* FIXME */	
 	g_return_if_fail (menu != NULL);
 
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
+	if (event != NULL) {
+		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
+				event->button, event->time);
+	} else {
+		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
+				menu_position_under_tree_view, treeview,
+				0, gtk_get_current_event_time ());
+
+		gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
+	}
 }
 
 /***** Callbacks *****/
@@ -1013,9 +1022,6 @@ click (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 {
 	GtkTreePath *path;
 	GtkTreeSelection *select;
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	struct session *s;
 
 	if (!event)
 		return FALSE;
@@ -1031,11 +1037,7 @@ click (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 			gtk_tree_path_free (path);
 		}
 
-		model = GTK_TREE_MODEL (NAVTREE (treeview)->model->sorted);
-		gtk_tree_model_get_iter (model, &iter, NAVTREE (treeview)->current_path);
-		gtk_tree_model_get (model, &iter, 2, &s, -1);
-		if (s != NULL)
-			navigation_context (treeview, s); /* FIXME */
+		show_context_menu (treeview, event);
 		return TRUE;
 	}
 
@@ -1046,29 +1048,39 @@ click (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 static gboolean
 declick (GtkWidget *treeview, GdkEventButton *e, gpointer data)
 {
+	gint position;
+
+	position = gtk_editable_get_position (GTK_EDITABLE (gui.text_entry));
 	gtk_widget_grab_focus (gui.text_entry);
-	gtk_editable_set_position (GTK_EDITABLE (gui.text_entry), -1);
+	gtk_editable_set_position (GTK_EDITABLE (gui.text_entry), position);
 	g_object_set (G_OBJECT (treeview), "can-focus", TRUE, NULL);
 	return FALSE;
 }
 
+static gboolean
+popup_menu (GtkWidget *treeview, gpointer user_data)
+{
+	show_context_menu (treeview, NULL);
+	return TRUE;
+}
+
 static void
 navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_data)
 {
-	GtkTreeIter iter, newiter;
-	GtkTreeModel *model, *store;
-	GtkTreeView *treeview;
-	GtkWidget *button;
-	gpointer *s;
-	session *sess;
+	GtkTreeIter   iter;
+	GtkTreeIter   newiter;
+	GtkTreeModel *model;
+	GtkTreeModel *store;
+	GtkWidget    *button;
+	gpointer     *s;
+	session      *sess;
 
 	if (gui.server_tree == NULL)
 		return;
 
-	treeview = GTK_TREE_VIEW (glade_xml_get_widget (gui.xml, "userlist"));
 
-	if (gui.server_tree->current_path != NULL)
-		navigation_model_path_deref (gui.server_tree->model, gui.server_tree->current_path);
+	if (gui.server_tree->current_rowref != NULL)
+		navigation_model_rowref_deref (gui.server_tree->current_rowref);
 
 	/* If find bar is open, hide it */
 	find_bar_close (FIND_BAR (gui.find_bar));
@@ -1078,14 +1090,17 @@ navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_dat
 	 */
 	if (gtk_tree_selection_get_selected (treeselection, &model, &iter) && gui.current_session) {
 		GtkWidget *menuitem;
+		GtkTreePath *path;
 
-		/* Update current_path. */
-		if (gui.server_tree->current_path) {
-			gtk_tree_path_free (gui.server_tree->current_path);
+		path = gtk_tree_model_get_path (model, &iter);
+
+		/* Update current_rowref. */
+		if (gui.server_tree->current_rowref) {
+			gtk_tree_row_reference_free (gui.server_tree->current_rowref);
 		}
 
-		gui.server_tree->current_path = gtk_tree_model_get_path (model, &iter);
-		navigation_model_path_ref (gui.tree_model, gui.server_tree->current_path);
+		gui.server_tree->current_rowref = gtk_tree_row_reference_new(model, path);
+		navigation_model_rowref_ref (gui.server_tree->current_rowref);
 		gtk_tree_selection_get_selected (treeselection, &model, &iter);
 
 		/* Get the session for the new selection. */
@@ -1101,7 +1116,8 @@ navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_dat
 		gui.current_session = sess;
 
 		/* Change the model in the userlist to the one for this session. */
-		gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (userlist_get_store (u, sess)));
+		gtk_tree_view_set_model (GTK_TREE_VIEW (gui.userlist),
+				         GTK_TREE_MODEL (userlist_get_store (u, sess)));
 
 		/* Set our nick. */
 		set_nickname (sess->server, NULL);
@@ -1131,7 +1147,6 @@ navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_dat
 		gtk_widget_set_sensitive (menuitem, sess->type != SESS_SERVER);
 		gtk_widget_set_sensitive (gui.userlist_toggle, sess->type == SESS_CHANNEL);
 
-
 		/* remove any icon that exists */
 		store = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
 		gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model), &newiter, &iter);
@@ -1142,6 +1157,12 @@ navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_dat
 		text_entry_set_current         (TEXT_ENTRY         (gui.text_entry),         sess);
 		status_bar_set_current         (STATUS_BAR         (gui.status_bar),         sess->server);
 
+		gtk_widget_grab_focus (gui.text_entry);
+		gtk_editable_set_position (GTK_EDITABLE (gui.text_entry), -1);
+
+		/* Set the label of the user list button */
+		userlist_set_user_button (u, sess);
+
 		/* Emit "focus tab" event */
 		plugin_emit_dummy_print (sess, "Focus Tab");
 	}
@@ -1150,16 +1171,23 @@ navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_dat
 static void
 row_expanded (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data)
 {
+	GtkTreePath *current_path;
+
+	if (NAVTREE (treeview)->current_rowref == NULL)
+		return;
+	current_path = gtk_tree_row_reference_get_path (NAVTREE (treeview)->current_rowref);
+
 	/* If we had something selected before the row was collapsed make sure it gets selected. */
-	if (NAVTREE (treeview)->current_path && gtk_tree_path_is_ancestor (path, NAVTREE (treeview)->current_path)) {
+	if (current_path && gtk_tree_path_is_ancestor (path, current_path)) {
 		GtkTreeSelection *selection;
 		selection = gtk_tree_view_get_selection (treeview);
 		g_signal_handler_block ((gpointer) selection, NAVTREE (treeview)->selection_changed_id);
-		gtk_tree_selection_select_path (selection, NAVTREE (treeview)->current_path);
+		gtk_tree_selection_select_path (selection, current_path);
 		g_signal_handler_unblock ((gpointer) selection, NAVTREE (treeview)->selection_changed_id);
 	}
 
 	navigation_tree_update_refs (NAVTREE (treeview));
+	gtk_tree_path_free(current_path);
 }
 
 static void
@@ -1168,8 +1196,6 @@ row_collapsed (GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * path, g
 	navigation_tree_update_refs (NAVTREE (treeview));
 }
 
-
-
 /********** NavModel **********/
 
 static void navigation_model_init       (NavModel * navmodel);
@@ -1285,15 +1311,14 @@ static gboolean
 navigation_model_create_new_channel_entry_iterate (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, struct session *data)
 {
 	struct session *s;
+	gint            position;
+
 	gtk_tree_model_get (model, iter, 2, &s, -1);
 	if (s->type == SESS_SERVER && s->server == data->server) {
 		GtkTreeIter child;
-		GtkTreeModelSort *sort;
 		GtkTreeRowReference *rowref;
 		GtkTreePath *path;
 
-		sort = GTK_TREE_MODEL_SORT (gui.tree_model->sorted);
-
 		gtk_tree_store_append (GTK_TREE_STORE (model), &child, iter);
 		/* Add the new channel to the store. By default connected is true. */
 		gtk_tree_store_set (GTK_TREE_STORE (model), &child, 1, data->channel, 2, data, 3, 0, 4, NULL, 5, 0, 6, TRUE, -1);
@@ -1302,10 +1327,9 @@ navigation_model_create_new_channel_entry_iterate (GtkTreeModel *model, GtkTreeP
 		rowref = gtk_tree_row_reference_new (model, path);
 		g_hash_table_insert (gui.tree_model->session_rows, (gpointer) data, (gpointer) rowref);
 
-		/* Set our nick. */
-		set_nickname (s->server, NULL);
-
+		position = gtk_editable_get_position (GTK_EDITABLE (gui.text_entry));
 		gtk_widget_grab_focus (gui.text_entry);
+		gtk_editable_set_position (GTK_EDITABLE (gui.text_entry), position);
 
 		gtk_tree_path_free (path);
 		return TRUE;
@@ -1396,10 +1420,7 @@ navigation_model_set_hilight_iterate (GtkTreeModel * model, GtkTreePath * path,
 
 	gtk_tree_model_get (model, iter, 2, &s, 3, &e, 5, &ref, -1);
 
-	/* FIXME Comment out the reference counting check here just to see if this is what is actually
-	 * breaking the icons.
-	 */
-	if (s == data) { /* && ref == 0) { */
+	if (s == data && ref == 0) {
 		struct session *sess = s;
 
 		if (sess->nick_said) {
@@ -1407,7 +1428,10 @@ navigation_model_set_hilight_iterate (GtkTreeModel * model, GtkTreePath * path,
 			return TRUE;
 		}
 		if (sess->msg_said && e < 2) {
-			gtk_tree_store_set (GTK_TREE_STORE (model), iter, 0, pix_msgsaid, 3, 2, -1);
+			if (sess->type != SESS_DIALOG)
+				gtk_tree_store_set (GTK_TREE_STORE (model), iter, 0, pix_msgsaid, 3, 2, -1);
+			else
+				gtk_tree_store_set (GTK_TREE_STORE (model), iter, 0, pix_nicksaid, 3, 3, -1);
 			return TRUE;
 		}
 		if (sess->new_data && e < 1) {
@@ -1431,36 +1455,52 @@ navigation_model_set_hilight (NavModel * model, struct session *sess)
 }
 
 void
-navigation_model_path_ref (NavModel * model, GtkTreePath * path)
+navigation_model_rowref_ref (GtkTreeRowReference * rowref)
 {
 	gint ref_count;
-	GtkTreeIter iter;
-	GtkTreePath *unsorted = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (model->sorted), path);
+	GtkTreeIter iter, childiter;
+	GtkTreeModel *model, *childmodel;
+	GtkTreePath *path;
 
-	gtk_tree_model_get_iter (GTK_TREE_MODEL (model->store), &iter, unsorted);
-	gtk_tree_model_get (GTK_TREE_MODEL (model->store), &iter, 5, &ref_count, -1);
-	gtk_tree_store_set (model->store, &iter, 5, ref_count + 1, -1);
+	model = gtk_tree_row_reference_get_model (rowref);
+	path = gtk_tree_row_reference_get_path (rowref);
+	childmodel = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
 
-	gtk_tree_path_free (unsorted);
+	gtk_tree_model_get_iter (model, &iter, path);
+	gtk_tree_model_get (model, &iter, 5, &ref_count, -1);
+	gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model), &childiter, &iter);
+	gtk_tree_store_set (GTK_TREE_STORE (childmodel), &childiter, 5, ref_count + 1, -1);
+	gtk_tree_path_free (path);
 }
 
 void
-navigation_model_path_deref (NavModel * model, GtkTreePath * path)
+navigation_model_rowref_deref (GtkTreeRowReference *rowref)
 {
 	gint ref_count;
 	GtkTreeIter iter;
-	GtkTreePath *unsorted = gtk_tree_model_sort_convert_path_to_child_path (GTK_TREE_MODEL_SORT (model->sorted), path);
+	GtkTreeModel *model;
+	GtkTreePath *path;
 
-	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (model->store), &iter, unsorted) == FALSE) {
-		g_critical ("path is invalid in navigation_model_path_deref\n");
+	model = gtk_tree_row_reference_get_model (rowref);
+	path = gtk_tree_row_reference_get_path (rowref);
+
+	if (gtk_tree_model_get_iter (model, &iter, path) == FALSE) {
+		g_critical ("path is invalid in navigation_model_rowref_deref\n");
 		return;
 	}
-	gtk_tree_model_get (GTK_TREE_MODEL (model->store), &iter, 5, &ref_count, -1);
 
-	if (ref_count > 0)
-		gtk_tree_store_set (model->store, &iter, 5, ref_count - 1, -1);
+	gtk_tree_model_get (model, &iter, 5, &ref_count, -1);
 
-	gtk_tree_path_free (unsorted);
+	if (ref_count > 0) {
+		GtkTreeModel *childmodel;
+		GtkTreeIter childiter;
+
+		childmodel = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
+		gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model), &childiter, &iter);
+		gtk_tree_store_set (GTK_TREE_STORE (childmodel), &childiter, 5, ref_count - 1, -1);
+	}
+
+	gtk_tree_path_free (path);
 }
 
 void
@@ -1518,7 +1558,7 @@ on_server_reconnect (GtkAction * action, gpointer data)
 {
 	session *s;
 
-	s = navigation_tree_get_selected_session ();
+	s = navigation_tree_get_selected_session (NULL);
 	if (s)
 		s->server->auto_reconnect (s->server, FALSE, -1);
 }
@@ -1526,11 +1566,32 @@ on_server_reconnect (GtkAction * action, gpointer data)
 static void
 on_server_disconnect (GtkAction * action, gpointer data)
 {
-	session *s;
+	GtkTreeView      *treeview;
+	GtkTreeSelection *select;
+	GtkTreeModel     *model;
+	GtkTreeModel     *store;
+	GtkTreeIter       iter;
+	GtkTreeIter       newiter;
+	session          *s;
 
-	s = navigation_tree_get_selected_session ();
-	if (s)
+	treeview = GTK_TREE_VIEW (gui.server_tree);
+	select = gtk_tree_view_get_selection (treeview);
+	if (!gtk_tree_selection_get_selected (select, &model, &iter))
+		return;
+
+	gtk_tree_model_get (model, &iter, 2, &s, -1);
+	if (s) {
 		s->server->disconnect (s, TRUE, -1);
+
+		/* disconnect call may have changed tree model, so we have to refresh
+		 * the model/iter here */
+		gtk_tree_selection_get_selected (select, &model, &iter);
+	}
+
+	/* Mark as disconnected */
+	store = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
+	gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model), &newiter, &iter);
+	gtk_tree_store_set (GTK_TREE_STORE (store), &newiter, 6, FALSE, -1);
 }
 
 static void
@@ -1538,7 +1599,7 @@ on_server_channel_list (GtkAction * action, gpointer data)
 {
 	session *s;
 
-	s = navigation_tree_get_selected_session ();
+	s = navigation_tree_get_selected_session (NULL);
 	if (s)
 		create_channel_list (s);
 }
@@ -1565,23 +1626,10 @@ on_close (GtkAction * action, gpointer data)
 	if (gtk_tree_selection_get_selected (select, &model, &iter)) {
 		gtk_tree_model_get (model, &iter, 2, &s, 6, &joined, -1);
 
-		if (s->type == SESS_CHANNEL && joined) {
-			GConfClient *client;
-			gchar *text;
-
-			client = gconf_client_get_default ();
-			text = gconf_client_get_string (client, "/apps/xchat/irc/partmsg", NULL);
-			if (text == NULL)
-				text = g_strdup (_("Ex-Chat"));
-			s->server->p_part (s->server, s->channel, text);
-			g_object_unref (client);
-			g_free (text);
-		} else if (s->type == SESS_SERVER && joined) {
+		if (s->type == SESS_SERVER) {
+			status_bar_remove_server (STATUS_BAR (gui.status_bar), s->server);
 			s->server->disconnect (s, TRUE, -1);
 		}
-
-		if (s->type == SESS_SERVER)
-			status_bar_remove_server (STATUS_BAR (gui.status_bar), s->server);
 		conversation_panel_remove_session (CONVERSATION_PANEL (gui.conversation_panel), s);
 		topic_label_remove_session        (TOPIC_LABEL        (gui.topic_label),        s);
 		text_entry_remove_session         (TEXT_ENTRY         (gui.text_entry),         s);
@@ -1592,22 +1640,7 @@ on_close (GtkAction * action, gpointer data)
 static void
 on_find (GtkAction * action, gpointer data)
 {
-	GtkWidget *widget;
-
-	widget = glade_xml_get_widget (gui.xml, "find hbox");
-	gtk_widget_show (widget);
-	widget = glade_xml_get_widget (gui.xml, "find entry");
-	gtk_widget_grab_focus (widget);
-}
-
-static void
-on_clear (GtkAction * action, gpointer data)
-{
-	session *s;
-
-	s = navigation_tree_get_selected_session ();
-	if (s)
-		conversation_panel_clear (CONVERSATION_PANEL (gui.conversation_panel), s);
+	find_bar_open (FIND_BAR (gui.find_bar));
 }
 
 static void
@@ -1624,7 +1657,7 @@ on_channel_leave (GtkAction * action, gpointer data)
 	if (gtk_tree_selection_get_selected (select, &model, &iter)) {
 		gtk_tree_model_get (model, &iter, 2, &s, -1);
 		store = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
-		if (s->type == SESS_CHANNEL) {
+		if ((s->type == SESS_CHANNEL) && (s->channel[0] != '\0')) {
 			GConfClient *client;
 			gchar *text;
 
@@ -1653,7 +1686,7 @@ on_server_autoconnect (GtkAction *action, gpointer data)
 	gboolean autoconnect, active;
 	ircnet *network;
 
-	sess = navigation_tree_get_selected_session ();
+	sess = navigation_tree_get_selected_session (NULL);
 	network = sess->server->network;
 	autoconnect = network->flags & FLAG_AUTO_CONNECT;
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action) );
@@ -1679,7 +1712,7 @@ on_channel_autojoin (GtkAction *action, gpointer data)
 	gboolean autojoin, active;
 	ircnet *network;
 
-	sess = navigation_tree_get_selected_session ();
+	sess = navigation_tree_get_selected_session (NULL);
 	network = sess->server->network;
 	autojoin = channel_is_autojoin (network, sess->channel);
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action) );
